function _typeof(obj) {
  "@babel/helpers - typeof";

  if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") {
    _typeof = function (obj) {
      return typeof obj;
    };
  } else {
    _typeof = function (obj) {
      return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
    };
  }

  return _typeof(obj);
}

var requestMethods = ['delete', 'get', 'head'];
var dataMethods = ['patch', 'post', 'put'];
var responseTypes = {
  arrayBuffer: '*/*',
  blob: '*/*',
  formData: 'multipart/form-data',
  json: 'application/json',
  text: 'text/*'
};
function FetchError(response, details) {
  this.name = 'FetchError';
  this.message = response.statusText;
  this.status = response.status;
  this.response = response;
  this.stack = new Error().stack;
}
FetchError.prototype = Object.create(Error.prototype);
FetchError.prototype.constructor = FetchError;
function factory(defaults, api) {
  api = api || typeof window !== 'undefined' && window || typeof global !== 'undefined' && global || typeof globalThis !== 'undefined' && globalThis;
  var fetch = api.fetch;
  var Headers = api.Headers;
  var URL = api.URL;
  var URLSearchParams = api.URLSearchParams;

  function makeRequest(url, options) {
    return fetch(url, options).then(function (res) {
      if (!res.ok) {
        if (res.headers.get('content-type').includes('application/json')) {
          return res.json().then(function (details) {
            throw new FetchError(res, details);
          });
        }

        throw new FetchError(res);
      }

      return res;
    });
  }

  function rek(url, options) {
    options = Object.assign({}, defaults, options);

    if (options.baseUrl) {
      url = new URL(url, options.baseUrl).href;
    }

    if (options.searchParams) {
      url = "".concat(url.split('?')[0], "?").concat(new URLSearchParams(options.searchParams));
    }

    var headers = options.headers = new Headers(Object.assign({}, defaults.headers, options.headers));
    var body = options.body;

    if (body && _typeof(body) === 'object' && typeof body.append !== 'function') {
      var contentType = headers.get('content-type');

      if (!contentType) {
        headers.set('content-type', contentType = 'application/json');
      }

      if (contentType.includes('application/json')) {
        options.body = JSON.stringify(body);
      }

      if (contentType.includes('application/x-www-form-urlencoded')) {
        options.body = new URLSearchParams(body);
      }
    }

    var obj = {
      run: function run() {
        return makeRequest(url, options);
      },
      then: function then(onResolved, onRejected) {
        return makeRequest(url, options).then(onResolved, onRejected);
      }
    };

    var _loop = function _loop(type) {
      obj[type] = function () {
        headers.set('accept', responseTypes[type]);
        return makeRequest(url, options).then(function (res) {
          return res[type]();
        });
      };
    };

    for (var type in responseTypes) {
      _loop(type);
    }

    return obj;
  }

  requestMethods.forEach(function (method) {
    rek[method] = function (url, options) {
      return rek(url, Object.assign({}, options, {
        method: method.toUpperCase()
      }));
    };
  });
  dataMethods.forEach(function (method) {
    rek[method] = function (url, body, options) {
      return rek(url, Object.assign({}, options, {
        body: body,
        method: method.toUpperCase()
      }));
    };
  });

  rek.extend = function (newDefaults, newApi) {
    return factory(newDefaults ? Object.assign({}, defaults, newDefaults) : defaults, newApi ? Object.assign({}, api, newApi) : api);
  };

  rek.factory = factory;

  rek.getArgs = function () {
    return [defaults, api];
  };

  return rek;
}

var browser = factory({
  credentials: 'same-origin'
});

export default browser;
//# sourceMappingURL=rek.esm.js.map
