import unfetch from 'unfetch';
import factory from './factory.mjs';

function URL(url, baseUrl) {
  this.href = baseUrl ? baseUrl + '/' + url : url;
}

function URLSearchParams(obj) {
  var str = '';

  if (typeof obj === 'string') {
    str = obj;
  } else {
    for (var key in obj) {
      str += "".concat(str ? '&' : '').concat(encodeURIComponent(key), "=").concat(encodeURIComponent(obj[key]));
    }
  }

  this._s = str;
}

URLSearchParams.prototype.toString = function () {
  return this._s;
};

function Headers(headers) {
  var _this = this;

  this.__h = {};

  for (var header in headers) {
    this.__h[header.toLowerCase()] = headers[header];
  }

  this.get = function (n) {
    return _this.__h[n.toLowerCase()];
  };

  this.set = function (n, v) {
    return _this.__h[n.toLowerCase()] = v;
  };
}

function fetch(url, options) {
  options.headers = options.headers.__h;
  return unfetch(url, options);
}

export default factory({
  credentials: 'same-origin'
}, {
  fetch: fetch,
  Headers: Headers,
  URL: URL,
  URLSearchParams: URLSearchParams
});
//# sourceMappingURL=unfetch.mjs.map